<?php
/**
 * Generates an array with abstracted browser information
 * Hook for t3lib_utility_Client::getBrowserInfo
 * 
 * @param	array		$params: Parameter to parse
 * @param 	object		$object: The parent object
 * @return	array		Contains keys "browser", "version", "system"
 */
function user_getBrowserInfo(array &$params, &$object) {
	$userAgent = trim($params['userAgent']);
	$browserInfo = array(
		'useragent' => $userAgent,
	);
	
	// Analyze the userAgent string
	// Declare known browsers to look for
	
	$known = array('msie', 'firefox', 'webkit', 'opera', 'netscape', 'konqueror',
			'gecko', 'chrome', 'safari', 'seamonkey', 'navigator', 'mosaic',
			'lynx', 'amaya', 'omniweb', 'avant', 'camino', 'flock', 'aol');
	$matches = array();
	
	$pattern = '#(?P<browser>'.join('|', $known).')[/ ]+(?P<version>[0-9]+(?:\.[0-9]+)?)#';
	// Find all phrases (or return empty array if none found)
	if (!preg_match_all($pattern, strtolower($userAgent), $matches)) {
		$browserInfo['browser'] = 'unknown';
		$browserInfo['version'] = '';
		$browserInfo['all'] = array();
	} else {
		// Since some UAs have more than one phrase (e.g Firefox has a Gecko phrase,
		// Opera 7,8 have a MSIE phrase), use the last one found (the right-most one
		// in the UA).  That's usually the most correct.
		// For IE use the first match as IE sends multiple MSIE with version, from higher to lower.
		$lastIndex = count($matches['browser']) - 1;
		// Decrease index for chrome, safari and opera for -1
		$decreaseKnown = array('chrome');
		if ($lastIndex > 0 && preg_match('#'.join('|', $decreaseKnown).'[/ ]+#', strtolower($userAgent))) {
			$lastIndex--;
		}
		// UserAgent has "version/"-Flag
		$versionFlagKnown = array('opera', 'safari');
		if (strpos(strtolower($userAgent), 'version/') !== FALSE && preg_match('#(?P<browser>'.join('|', $versionFlagKnown).')[/ ]+#', strtolower($userAgent))) {
			$pattern = '#(?P<browser>version)[/ ]+(?P<version>[0-9]+(?:\.[0-9]+)?)#';
			if (preg_match_all($pattern, strtolower($userAgent), $vMatches)) {
				$matches['version'][$lastIndex] = $vMatches['version'][0];
			}
		}
		$browserInfo['browser'] = $matches['browser'][$lastIndex];
		$browserInfo['version'] = $browserInfo['browser'] === 'msie' ? $matches['version'][0] : $matches['version'][$lastIndex];
		// But return all parsed browsers / version in an extra array
		for ($i = 0; $i <= $lastIndex; $i++) {
			if (!isset($browserInfo['all'][$matches['browser'][$i]])) {
				$browserInfo['all'][$matches['browser'][$i]] = $matches['version'][$i];
			}
		}
		// Replace gecko build date with version given by rv
		if (isset($browserInfo['all']['gecko'])) {
			preg_match_all('/rv:([0-9\.]*)/', strtolower($userAgent), $version);
			if ($version[1][0]) {
				$browserInfo['all']['gecko'] = $version[1][0];
			}
		}
	}
	
	// Microsoft Documentation about Platform tokens: http://msdn.microsoft.com/en-us/library/ms537503(VS.85).aspx
	// 'system' is deprecated, use 'all_systems' (array) in future!
	$browserInfo['system'] = '';
	$browserInfo['all_systems'] = array();
	if (strstr($userAgent, 'Win')) {
		// windows
		if (strstr($userAgent, 'Windows NT 6.3')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'win81';
			$browserInfo['all_systems'][] = 'win8';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Windows NT 6.2')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'win8';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Windows NT 6.1')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'win7';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Windows NT 6.0')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'winVista';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Windows NT 5.1')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'winXP';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Windows NT 5.0')) {
			$browserInfo['system'] = 'winNT'; // backwards compatible
			$browserInfo['all_systems'][] = 'win2k';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Win98') || strstr($userAgent, 'Windows 98')) {
			$browserInfo['system'] = 'win98';
			$browserInfo['all_systems'][] = 'win98';
		} elseif (strstr($userAgent, 'Win95') || strstr($userAgent, 'Windows 95')) {
			$browserInfo['system'] = 'win95';
			$browserInfo['all_systems'][] = 'win95';
		} elseif (strstr($userAgent, 'WinNT') || strstr($userAgent, 'Windows NT')) {
			$browserInfo['system'] = 'winNT';
			$browserInfo['all_systems'][] = 'winNT';
		} elseif (strstr($userAgent, 'Win16') || strstr($userAgent, 'Windows 311')) {
			$browserInfo['system'] = 'win311';
			$browserInfo['all_systems'][] = 'win311';
		}
	} elseif (strstr($userAgent, 'Mac')) {
		if (strstr($userAgent, 'iPad') || strstr($userAgent, 'iPhone') || strstr($userAgent, 'iPod')) {
			$browserInfo['system'] = 'mac'; // backwards compatible
			$browserInfo['all_systems'][] = 'iOS';
			$browserInfo['all_systems'][] = 'mac';
		} else {
			$browserInfo['system'] = 'mac';
			$browserInfo['all_systems'][] = 'mac';
		}
		// unixes
	} elseif (strstr($userAgent, 'Android')) {
		$browserInfo['system'] = 'linux'; // backwards compatible
		$browserInfo['all_systems'][] = 'android';
		$browserInfo['all_systems'][] = 'linux';
	} elseif (strstr($userAgent, 'Linux')) {
		$browserInfo['system'] = 'linux';
		$browserInfo['all_systems'][] = 'linux';
	} elseif (strstr($userAgent, 'BSD')) {
		$browserInfo['system'] = 'unix_bsd';
		$browserInfo['all_systems'][] = 'unix_bsd';
	} elseif (strstr($userAgent, 'SGI') && strstr($userAgent, ' IRIX ')) {
		$browserInfo['system'] = 'unix_sgi';
		$browserInfo['all_systems'][] = 'unix_sgi';
	} elseif (strstr($userAgent, ' SunOS ')) {
		$browserInfo['system'] = 'unix_sun';
		$browserInfo['all_systems'][] = 'unix_sun';
	} elseif (strstr($userAgent, ' HP-UX ')) {
		$browserInfo['system'] = 'unix_hp';
		$browserInfo['all_systems'][] = 'unix_hp';
	} elseif (strstr($userAgent, 'CrOS')) {
		$browserInfo['system'] = 'linux';
		$browserInfo['all_systems'][] = 'chrome';
		$browserInfo['all_systems'][] = 'linux';
	}
	
	return $browserInfo;
	
}